
#include "PulseView.h"

#include <new.h>
#include <stdio.h>
#include <stdlib.h>
#include <Application.h>
#include <FindDirectory.h>
#include <Path.h>
#include <Screen.h>
#include <Window.h>

const char* AppSig = "application/x-vnd.CS-MiniPulse";
const char* AboutMiniPulse = "MiniPulse 2.2 by Chuck Sanders\n"
                             "http://members.home.net/chucksanders/besw.html";
const char* SettingsFileName = "MiniPulse_settings";

class TPulseWindow : public BWindow {
	private:
	
		void moveToRightPlace (BRect screenBounds)
		{	float xMax, yMax;
		
			xMax = yMax = 2000000;
		
			BRect f = Frame();
			float x = f.left;
			float y = f.top;
			
			float diff = xMax - x;
			xMax = screenBounds.right - PROGRESS_WIDTH - 3;
			if (diff < 50)
				x = xMax - diff;
			if (x > xMax)
				x = xMax;
			if (x < 0)
				x = 0;
			
			diff = yMax - y;
			yMax = screenBounds.bottom - 3*PROGRESS_HEIGHT - 4;
			if (diff < 50)
				y = yMax - diff;
			if (y > yMax)
				y = yMax;
			if (y < 0)
				y = 0;
			
			MoveTo (x, y);
		}
		
	public:
		TPulseWindow()
			: BWindow(BRect (0, 0, 1, 1), "BP6 CPU Monitor", B_FLOATING_WINDOW_LOOK,
			  B_NORMAL_WINDOW_FEEL, B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
		{
		
			SetSizeLimits (1, 9999, 1, 9999);
			ResizeTo (TEMP_WIDTH+PROGRESS_WIDTH+FAN_WIDTH, 3*PROGRESS_HEIGHT + 1);
			
			float x = 1000000, y = 1000000; // this should get the lower-right corner
			
			
			// Read window position from preferences file
			BPath settingsPath;
			if (B_OK == find_directory (B_USER_SETTINGS_DIRECTORY, &settingsPath, false)) {
				settingsPath.Append (SettingsFileName);
				BFile *settingsFile = new BFile (settingsPath.Path(), B_READ_ONLY);
				if (B_NO_ERROR == settingsFile->InitCheck()) {
					if (settingsFile->Read (&x, sizeof(x)) != sizeof(x))
						x = 1000000; // reset to default on error
					if (settingsFile->Read (&y, sizeof(y)) != sizeof(y))
						y = 1000000; // reset to default on error
				}
				delete settingsFile;
			}
			
			AddChild (new TPulseView(BRect (0, 0, TEMP_WIDTH+PROGRESS_WIDTH+FAN_WIDTH,3*PROGRESS_HEIGHT + 1)));
			
			MoveTo (x, y);
			moveToRightPlace (BScreen(B_MAIN_SCREEN_ID).Frame());
			Show();
		}
		
		virtual void ScreenChanged (BRect frame, color_space /* mode */)
		{
			moveToRightPlace (frame);
		}
		
		bool QuitRequested()
		{
			// Save window position...
			BPath settingsPath;
			if (B_OK == find_directory (B_USER_SETTINGS_DIRECTORY, &settingsPath, false)) {
				settingsPath.Append (SettingsFileName);
				BFile *settingsFile = new BFile (settingsPath.Path(), B_WRITE_ONLY | B_CREATE_FILE);
				if (B_NO_ERROR == settingsFile->InitCheck()) {
					BRect r = Frame();
					float x = r.left;
					float y = r.top;
					settingsFile->Write (&x, sizeof(x));
					settingsFile->Write (&y, sizeof(y));
				}
				delete settingsFile;
			}
			
			be_app->PostMessage (B_QUIT_REQUESTED);
			return true;
		}
};

int main()
{
	// Create the app
	BApplication app (AppSig);
	
	// Open up a window
	new TPulseWindow();
	
	// Start the app running; when it's done, we go away.
	app.Run();
	return EXIT_SUCCESS;
}
